      program flexout2srm

C* This program reads in the FLEXPART output file and
C* writes station results to the source-receptor matrix
C* archive for further postprocessing

CPS Petra Seibert, Dec 2006, on base of a programm by Gerhard Wotawa
CPS Version 1.0
CPS This programme reads the FLEXPART bwd output files and stores them 
CPS in an intermediate array cfor with dimensions number of species
CPS and number of output times during the lifetime (backtracking duration)
CPS of the substance simulated. Then data are written out in reverse
CPS order to FLEXPART, i.e. one file for each receptor time instead
CPS of on file for each source time!
CPS You have to set some parameters in the f2s_nest.incf file!
CPS Parameters lsingle and lsum (to be set below) control output of
CPS all single fields, and/or fields aggregated to 24 hour receptor
CPS and 1x1 degree source resolution.

CPS Previous version was 2007-01-31 16:01

CPS 26 March 2007: Correction of bug in writeforsum 
CPS when the output grid coordinates are calculated, we have to use
CPS xpos = outlon0 + (ix-2)*dxout, ypos = outlat0 + (jy-2)*dyout
CPS (not ix-1 etc)
 
CPS 13 April 2007: average instead of sum over receptor times
 
	logical lsingle, lsum
	parameter (lsingle=.true.,lsum=.true.)
      parameter (ilfndate=80,ilpath=200)
c	maximum dimensions of Flexpart fields

	character path*(ilpath), fndate*(ilfndate)
	
	include 'f2s_nest.incf'

c Read Control file
      open (40,file='control',status='old')
	print*
      print*, '*** file "control" opened'
	print*

      read (40,'(a)') path
      lenp = index(path,' ')-1
      read (40,*)
      read (40,*) klev
      read (40,*) xmulti
      read (40,'(a2)') mstamp
      read (40,'(a2)') rstamp
      read (40,'(a4)') centerid
      close (40)


      call rdheader (path)
	
c Start section with main loop over all the FLEXPART output files
      numtime = 0
	
c     Open dates files

      open (41,file = path(1:lenp)//'dates', status='old')
	print*
      print*, '*** file "dates" opened'
	print*

c     Begin loop over all output files

25    read (41,'(a)',end=100) fndate
        numtime = numtime + 1
        call rddata (path,fndate) ! read and process one Flexpart file
      goto 25 ! end output file loop
	  
100   close( 41) ! close dates file

c     Start output
	if (lsingle) call writefor
	
	if (lsum) then
	  if (dxout .ne. 0.5 .or. dyout .ne. 0.5) then
	    print*, 'summing mode only prepare for 0.5 deg output'
	    stop 'summing not for 0.5 deg'
	  endif  
	  call writesumfor (06)
	endif

	stop
      end

 	subroutine rdheader (path)
c--------------------------------------------------------------------------	
c	read FLEXPART V6.2 header
c     Petra Seibert, Dec 2006 after existing code from GW and ASt
c--------------------------------------------------------------------------	

	include 'f2s_nest.incf'

	character path*(*)
c--------------------------------------------------------------------------	

      lenp = index(path,' ')-1
      open (40,file=path(1:lenp)//'header_nest', form='unformatted',
     &         status='old')
     
      read (40) ibdate,ibtime
      julbeg=juldate(ibdate,ibtime)
      write (*,*) 'START OF (BACKWARD) SIMULATION: ',ibdate,ibtime
      
	read (40) loutstep,loutaver,loutsample
      write (*,*) 'TIMESTEP OF OUTPUT:           ',loutstep
	
      read (40) outlon0,outlat0,numxgrid,numygrid,dxout,dyout
      write (*,*) 'OUTPUT GRID (X0,Y0,NX,NY,DX,DY): ',
     &            outlon0,outlat0,numxgrid,numygrid,dxout,dyout
      if (numxgrid .gt. nxmax) stop 'increase nxmax'
      if (numygrid .gt. nymax) stop 'increase nymax'
     
      read (40) numzgrid, (outheight(i),i=1,numzgrid)
      write (*,*) 'VERTICAL LAYERS: ',numzgrid, (outheight(i),
     &             i=1,numzgrid)
      if (numzgrid .gt. nzmax) then 
	  print*, 'increase nzmax'
        print*, 'note that presently only numzgrid=1 implemented'
	  stop
	endif
	  
c	calculate inverse of grid cell volumes
      vol0 =  dxout*111.e3 * dyout*111.e3 * outheight(1) 
      do 50 jy = 1,numygrid
	  volinv(jy) = 1. / (vol0 * cosd( outlat0+dyout*(jy-0.5) ) )
50	continue	        
	
	read (40)
      read (40) nspec
      nspec = nspec/3
      write (*,'(a,i2)') 'Number of Species:           ',nspec

      do 10 i = 1,nspec
        read (40)
        read (40)
10    read (40)
      read (40) numpoint
      write (*,'(a,i5)') 'Number of release locations: ',numpoint

      do 20 i=1,numpoint ! loop releases

        read (40) irelstart(i),irelend(i)
        read (40) xp1,yp1,xp2,yp2,zpoint1(i),zpoint2(i)
        xlon(i) = xp1
        ylat(i) = yp1
        read (40) npart(i),kind(i)
        read (40) compoint(i)
	  
	  if (i .eq. 1) then
	    irelstep = abs( irelstart(i) - irelend(i) )
	  endif  
	  if (i .ge. 2) then
	   if (irelstep .ne. abs(irelstart(i) - irelend(i)) )
     &    then
            print*, 'Your releases are of variable duration'
		stop ' Variable releaese duration not implemented'
	   endif	
         if (irelstep .ne. abs(irelstart(i) - irelstart(i-1)) ) then
            print*, 'Your release intervals are not equal to ' //
     &		'release durations'
		stop ' Inconsistent release duration and interval'
	   endif	
        endif
	  
	  compoint(i) = stationid
        lenc(i) = index(compoint(i),' ')-1

c	  calculate begin and end of release in absolute date
c       note that start is always the earlier time, even in bwd run

        julstop = julbeg + dble( irelend(i) ) / 86400.d0
        call caldate( julstop, jmdstop(i), ihstop(i) )
        ihstop(i) = ihstop(i)/10000
	 
        julstart = julbeg + dble( irelstart(i) )/ 86400.d0
        call caldate( julstart, jmdstart(i), ihstart(i) )
        ihstart(i) = ihstart(i)/10000
 
CPSK    add centerid in output fname
        write (timestamp,'(i8.8,i2.2)') jmdstop(i),ihstop(i)
        confile(i) = compoint(i)(1:lenc(i))//'.'//mstamp//'.'//
     &               timestamp//'.'//centerid//'.'//rstamp//'.srm'
        itimeshift(i) = nint( (julstop-julbeg)*86400.d0 )

        do 18 j=1,nspec
          read (40)
          read (40)
18      read (40) xmass(i,j)

        srsmmass(i) = xmass(i,1)
        if (i .eq. 1) then
          write(*,*)
          write(*,'(a)') '  # STAT   LON     LAT      H1  H2   '//
     &     '  START    STOP      MASS    NPART'
          write(*,*)
        endif

        write (*,15) i,compoint(i),xp1,yp1,zpoint1(i),zpoint2(i),
     &      irelstart(i),irelend(i),srsmmass(i),npart(i)
15      format(i3,1x,a5,2f8.3,2f6.0,2i8,2x,e8.2,i8)

20    continue ! end loop releases

      read (40) method,lsubgrid,lconvection
      read (40) nageclass, ( lage(i), i=1,nageclass )
	if (nageclass .ne. 1) then
	  print*, 'this programme works only with 1 age class'
	  stop
	endif  
	
	do 21 ix = 0,numxgrid-1
21      read (40) 
      close(40) ! close file 'header'

      return
	end


      subroutine rddata (path,fndate)
c--------------------------------------------------------------------------	
c     This subroutine reads and processes one Flexpart 6.2 output file
c     Petra Seibert, Dec 2006 after existing code from GW and ASt
c--------------------------------------------------------------------------	
	
	include 'f2s_nest.incf'
	character path*(*),fndate*(*)
	save itimeb, itimee
	
      lenp = index(path,' ')-1
      lend=index(fndate,' ')-1
	
c     Open data file
      iunit = 40	
      open (iunit, file=path(1:lenp)//'grid_time_nest_'//fndate(1:lend),
     &      form='unformatted',status='old')
c      write(*,'(a)') 'Open data file grid_time_nest'//fndate(1:lend)


C     Read header section of data file

      read (iunit) itime
      write (*,'(a,1x,i12,i4)') 
     &  'Open data file grid_time_'//fndate(1:lend),itime,numtime

      if (itime .lt. 0) then ! bwd run, nspec=numpoint
        nspec = numpoint
c        write (*,*) 'nspec,numpoint=',nspec,numpoint
      endif

      if (numtime .eq. 1) then
	  itimeb = itime
	  call initr (cfor,nxmax*nymax*nzmax*maxspec*itxx, 0.)
	  if (lage(1) .ne. 999999999) then
c	    If ageclasses are used, the number of source time intervals
c         is given by iage. Here we assume that only the first age class 
c         is used! If the release is over a finite time, we have to skil
c         the oldest source interval as it will be incomplete!
c         Note that this is is not a general formulation!
	    itmax = lage(1) / ioutstep - 1
	    print*, 'itmax set to', itmax
	    if (itmax .gt. itxx) stop 'increase itxx'
	  else
	    itmax = itxx   
	  endif  	
      endif

      if (numtime .eq. 2) then
        itimee = itime
	  joutstep = abs(itimeb-itimee)
        idhours = joutstep/3600
	  if (mod(joutstep,3600) .ne. 0) then
	    print*, 'your output step is not in full hours'
	    print*, joutstep,itimeb, itimee
	    stop 'output step not implemented'
	  endif  
	  if (ioutstep .ne. joutstep) then
	    print*, 'output step found is ', joutstep
	    print*, 'output step ioutstep is', ioutstep
	    print*, 'correct your parameter ioutstep'
	    stop 'ioutstep not matching data'
	  endif  
c     note that the length of backtracking could be defined also differently!	
	  lenback = itmax * ioutstep / 3600 ! length of backtracking in hrs
	  print*, 'itmax,lage,ioutstep = ', itmax, lage(1),ioutstep
      endif

c	Initialise concentrations
	call initr (wetgrid,nxmax*nymax*maxspec, 0.)
	call initr (drygrid,nxmax*nymax*maxspec, 0.)
	call initr (concgrid,nxmax*nymax*nzmax*maxspec*maxageclass, 0.)
	
      do 10 ispec = 1,nspec
        do 10 iage = 1,nageclass
	  
c	    print*,'reading ispec,iage',ispec,iage
	
c	    read wet depo
c	    print*, 'reading wetgrid'
	    call rdfield2 (wetgrid, numxgrid,numygrid,ispec,
     &	             nxmax,nymax,maxspec, iunit)
c	    read dry depo
c	    print*, 'reading drygrid'
	    call rdfield2 (drygrid, numxgrid,numygrid,ispec,
     &	             nxmax,nymax,maxspec, iunit)

c	    read conc 
	print*,'ispec,iage',ispec,iage

	call rdconc (itime,concgrid,drygrid, numxgrid,numygrid,numzgrid,
     &	ispec,iage,  nxmax,nymax,nzmax,maxspec,maxageclass, iunit)

10    continue ! end main data reading loops (species and age)
	close (iunit)


c	assign concentrations to the box of data that we really need
c	as this box contains FOR (fields of regard) in CTBTO terminology,
c     we call this variable cfor

      do 30 ispec = 1,nspec
c	  find the time index of ispec at present itime
c	  how old is species ispec now? we count from end of the release
	  jage = irelend(ispec) - itime ! this is positive for bwd run
	  it = jage / ioutstep 

	  if ( jage .gt. lage(1) ) then
c	    normally we would ignore this species
c         but if irelstep<ioutstep, a species can still be in the 
c         output up to one interval later
	    factoutrel = float(ioutstep)/float(irelstep)
	    ifactoutrel = nint( factoutrel )
	    if (ifactoutrel .gt. 1) then ! more than one species per output
	       
            if ( abs( factoutrel-float(ifactoutrel) ) .lt. 1.e-5 ) then
              print*, 'your output and release time  steps are not '//
     &        'multiples of each other, redo your FLEXPART simulations'
             stop 'factoutrel not good'
		endif
		 
		ioverage = irelstep * (ifactoutrel - 1)
		if (jage - lage(1) .le. ioverage) then
		  it = (jage - ioverage) / ioutstep
		endif  
	    endif	  
	  endif
	   
	  if (it .ge. 1 .and. it .le. itmax) then
	    ivalid = 0
          do 20 iage = 1,nageclass
	     do 20 kz = 1, nzmax
		do 20 jy = 1, nymax
		 do 20 ix = 1, nxmax
c              assign to box and add up ageclasses (pro forma)		 
		   cfor(ix,jy,ispec,it) =! cfor(ix,jy,ispec,it) + 
     &		   concgrid(ix,jy,kz,ispec,iage)*xmulti*volinv(jy)
               if (cfor(ix,jy,ispec,it) .gt. 0.) ivalid = ivalid + 1
20        continue
	  endif
	  write(*,901)ispec,irelend(ispec),itime,jage,it,itmax,ivalid
901     format (i3,3i12,3i6)	  
30    continue	
	return
	end

	subroutine initr (a,ix, val)
c---------------------------------------------------
c	initialise real array of dim ix with value val
c     PS, Dec 2006
c---------------------------------------------------
	real a(ix)
	
      do i=1,ix
        a(i) = val
	enddo
	return
	end  

	subroutine rdfield2 (grid, numxgrid,numygrid,ispec,
     &	nxmax,nymax,maxspec, iunit)
c---------------------------------------------------
c	read a FLEXPART 6.2 2-d field from iunit into grid
c     PS, Dec 2006
c---------------------------------------------------

	real grid(nxmax,nymax,maxspec)
	
      read (iunit) lval
      if (lval .eq. 1) then !sparse matrix

10      read (iunit) n,xx
          if (n .lt. 0) goto 50
          jy = n/numxgrid
          ix = n-numxgrid*jy
          ix = ix+1
          jy = jy+1
          grid(ix,jy,ispec) = xx
        goto 10
	    
      else
	
        do 20 ix = 1,numxgrid
20        read (iunit) (grid(ix,jy,ispec),jy=1,numygrid)
      
	endif

50    continue
	return
	end

      subroutine rdconc (itime,grid,dumgrid, numxgrid,numygrid,numzgrid,
     &  ispec,iage, nxmax,nymax,nzmax,maxspec,maxageclass, iunit)
c---------------------------------------------------
c	read a FLEXPART 6.2 concentration field from iunit into grid
c     PS, Dec 2006
c---------------------------------------------------

	real grid(nxmax,nymax,nzmax,maxspec,maxageclass)
	real dumgrid(nxmax,nymax,maxspec)

	nval = 0
c	read concentrations
      read (iunit) lval
      if (lval .eq. 1) then !sparse matrix
	  
10      read (iunit) n,xx ! ,yy no sigmas in nest!!!
          if (n .lt. 0) goto 50
          kz = n/(numxgrid*numygrid)
          jy = (n-kz*numxgrid*numygrid)/numxgrid
          ix = n - numxgrid*numygrid*kz - numxgrid*jy
          ix = ix+1
          jy = jy+1
          grid(ix,jy,kz,ispec,iage) = xx
c          congridsigma(ix,jy,kz,k)=yy ! here we skip the sigmas
	    nval = nval + 1
        goto 10

      else

        do 20 kz=1,numzgrid
          do 20 ix=1,numxgrid
            read (iunit) (grid(ix,jy,kz,ispec,iage),
C no sigmas in nest!!  &                      dumgrid(ix,jy,1),
     &       jy=1,numygrid)
20      continue

      endif

        
50	continue
	return
	end

	subroutine writefor
c---------------------------------------------------
c	write out all the FORs
c     PS, Dec 2006
c---------------------------------------------------
	
	include 'f2s_nest.incf'

	do 100 ispec = 1, nspec
 
        lenf=index(confile(ispec),' ')-1
        open (56,file=confile(ispec)(1:lenf))

c       Determine length of available backtracking for this species
        iendtime = irelend(ispec) - itmax*ioutstep
	  if (iendtime .lt. itime) then
	    iendtime2 = itime
	    itx = (irelend(ispec) - iendtime2 ) / ioutstep
	  else
	    itx = itmax    
	  endif
	  if (itx .gt. itxx) stop 'increase itxx'
	  
	  print*, ispec, itx,(irelend(ispec) - iendtime2 ) / ioutstep

c	  Write header
        lenb = itx*idhours
        write (56,910) xlon(ispec), ylat(ispec), jmdstart(ispec),
     &    ihstart(ispec), jmdstop(ispec), ihstop(ispec), 
     &    srsmmass(ispec), lenb, idhours, idhours, dxout, dyout,  
     &    '"'//compoint(ispec)(1:lenc(ispec))//'"',
     &    outlon0, outlat0, numxgrid, numygrid 
     

910     format (f7.2,1x,f6.2,1x,i8.8,1x,i2.2,1x,i8.8,1x,i2.2,
     &    1x,1p,e8.2,0p,1x,i5,i3,i3,2f6.3,1x,a,1x,2(f6.2,1x),2(i4,1x))

	  print*,'Writing file ',confile(ispec)(1:lenf)

c	  this version is for original Flexpart resolution
        do 50 it = 1,itx
          do 50 jy = 1,numygrid
            do 50 ix = 1,numxgrid
              if (cfor(ix,jy,ispec,it) .gt. 0.) then
                xpos = outlon0 + (ix-1)*dxout
                ypos = outlat0 + (jy-1)*dyout
                write (56,920) ypos,xpos,it,cfor(ix,jy,ispec,it)
920             format(f7.3,1x,f9.3,i4,1p,e15.7)
              endif
50      continue

        close (56) ! close output file
	  
100   continue ! end species loop

  	
	return
	end

	subroutine writesumfor (jhour)
c---------------------------------------------------
c	write out all the FORs 
c     sum mode for comparability with CTBTO IDC FORs:
c     sum into 1 deg, 1 receptor-day FORs
c     PS, Dec 2006
c---------------------------------------------------
	
	include 'f2s_nest.incf'

	do 100 ispec = 1, nspec
	
	  if (ihstop(ispec) .ne. jhour) goto 100
c       do everything only if at the end of a whole day at given hour
c       ispec is the last species in the averaging interval

	  jspec = ispec - 24/idhours + 1  
c       jspec is the first species in the averaging interval
	  print*,ispec,ihstop(ispec),jspec

	  if (jspec .le. 0) goto 100 ! incomplete interval	  
 
        lenf=index(confile(ispec),' ')-1
	  if (rstamp .eq. 'sn') confile(ispec)(lenf-4:lenf-4) = 'u'
c       replace runstamp 'sn' by 'su'
	  
        open (56,file=confile(ispec)(1:lenf))

c       Determine length of available backtracking for this species
c       we need to use the first species, this is cut off first
        iendtime = irelend(jspec) - itmax*ioutstep
	  iendtime2 = itime
	  if (iendtime .lt. itime) then
	    itx = (irelend(jspec) - iendtime2) / ioutstep - 1
	  endif  
	  if (lage(1) .ne. 999999999) then ! age classes are used, need to take 
c         care that earliest source interval is fully covered!	  
	    if (irelend(jspec) - iendtime2 .gt. lage(1)) then 
	      ! limit itx so that earliest source interval is fully covered!
	      itx2 = (irelend(ispec) - iendtime2 ) / ioutstep - 1
		if (itx2 .lt. itx) itx = itx2
	    endif	
	  endif
	
	  if (itx .gt. itxx) stop 'increase itxx'
	  
	  print*,'ispec,itx,..', ispec, itx,lage(1),
     &	  (irelend(jspec) - iendtime2 ) / ioutstep,
     &	  (irelend(ispec) - iendtime2 ) / ioutstep

c	  Write header
        lenb = itx*idhours
        write (56,910) xlon(ispec), ylat(ispec), jmdstart(jspec),
     &    ihstart(jspec), jmdstop(ispec), ihstop(ispec), 
     &    srsmmass(ispec), lenb, idhours, idhours, 2*dxout, 2*dyout,  
     &    '"'//compoint(ispec)(1:lenc(ispec))//'"',
     &    outlon0, outlat0, numxgrid/2, numygrid/2 
     

910     format (f7.2,1x,f6.2,1x,i8.8,1x,i2.2,1x,i8.8,1x,i2.2,
     &    1x,1p,e8.2,0p,1x,i5,i3,i3,2f6.3,1x,a,1x,2(f6.2,1x),2(i4,1x))

	  print*,'Writing file sum',confile(ispec)(1:lenf)


c     Average data of one release day

	icount = 0
	do 30 it = 1, itx
        do 30 is = jspec,ispec-1 ! ispec is already assigned to cfor
        print*,'Summing',jmdstart(is),ihstart(is),jmdstop(is),ihstop(is)
	    ispdel = ispec - is
	    itdel = ( irelstart(ispec) - irelstart(is) ) / ioutstep
	    ii = it + itdel
	    print*,it,is,ispdel,itdel,ii
	    if (ii .gt. itxx) goto 30
	    icount = icount + 1
           do 30 jy = 1,numygrid
            do 30 ix = 1,numxgrid
              cfor(ix,jy,jspec,it) = cfor(ix,jy,jspec,it) + 
     &		  cfor(ix,jy,is,ii)
30    continue
      xcnt = 1./icount

c	  this version is for 1 deg resolution (with 0.5 deg raw data)
        do 50 it = 1,itx
          do 50 jy = 2,numygrid,2
            do 50 ix = 2,numxgrid,2
		  cf = 0.25 * xcnt * ( 
     &	       cfor(ix,jy,jspec,it)   + cfor(ix-1,jy,jspec,it)
     &	     + cfor(ix,jy-1,jspec,it) + cfor(ix-1,jy-1,jspec,it) )
              if (cf .gt. 0.) then
                xpos = outlon0 + (ix-2)*2.*dxout
                ypos = outlat0 + (jy-2)*2.*dyout
                write (56,920) ypos,xpos,it,cf
920             format(f7.3,1x,f9.3,i4,1p,e15.7)
              endif
50      continue

        close (56) ! close output file
	  
100   continue ! end species loop

  	
	return
	end
	
	include 'juldate.f'
	include 'caldate.f'
